from typing import Any
from CxAdmin.api.cxItem import CxItem


class CxFlows(CxItem):
    def getFlows(self) -> list[dict[str, Any]]:
        flowsJson: list[dict[str, Any]] = self._httpClient.get(self._path).json()[
            "result"
        ]
        # flows = [CxFlow.from_json(flowJson) for flowJson in flowsJson]
        # return flows
        return flowsJson

    def getFlow(self, flowId: str) -> Any:
        raise NotImplementedError()

    def get(self) -> Any:
        return self.getFlows()



from CxAdmin.api.cxLists import CxLists
from CxAdmin.api.cxQueues import CxQueues
from CxAdmin.api.cxStatistics import CxStatistics
from CxAdmin.api.cxEnvironment import CxEnvironment
from CxAdmin.api.cxFlows import CxFlows
from CxAdmin.api.cxUsers import CxUsers
from CxAdmin.api.cxGroups import CxGroups

from CxAdmin.api.http.httpclient import HTTPClient
from CxAdmin.api.http.httpClientModel import HTTPClientModel

import json


class Cx:
    __BASE_URL: str
    __API_KEY: str
    __API_SECRET: str
    __TENANT_ID: str
    __TENANT_URL: str

    __httpClient: HTTPClientModel

    environment: CxEnvironment
    flows: CxFlows
    lists: CxLists
    queues: CxQueues
    users: CxUsers
    groups: CxGroups

    statistics: CxStatistics

    def __init__(self, baseURL: str, apiKey: str, apiSecret: str, tenantID: str):
        self.__BASE_URL = baseURL  # type: ignore
        self.__API_KEY = apiKey  # type: ignore
        self.__API_SECRET = apiSecret  # type: ignore
        self.__TENANT_ID = tenantID  # type: ignore
        self.__TENANT_URL = f"{self.__BASE_URL}/v1/tenants/{self.__TENANT_ID}"  # type: ignore

        self.__httpClient = HTTPClient(
            basePath=self.__TENANT_URL,
            token=self.__getToken(),
        )

        self.environment = CxEnvironment(self.__httpClient, "")
        self.flows = CxFlows(self.__httpClient, "/flows")
        self.lists = CxLists(self.__httpClient, "/lists")
        self.queues = CxQueues(self.__httpClient, "/queues")
        self.users = CxUsers(self.__httpClient, "/users")
        self.groups = CxGroups(self.__httpClient, "/groups")

        self.statistics = CxStatistics(self.__httpClient, "")

    @staticmethod
    def fromConfigFile(configFilePath: str) -> "Cx":
        config: dict[str, str]
        with open(file=configFilePath, mode="r") as file:
            config = json.load(file)

        return Cx(
            baseURL=config["baseURL"],
            apiKey=config["apiKey"],
            apiSecret=config["apiSecret"],
            tenantID=config["tenantID"],
        )

    def __getToken(self) -> str:
        token = HTTPClient.getToken(
            self.__BASE_URL,
            self.__API_KEY,
            self.__API_SECRET,
            self.__TENANT_ID,
        )

        return token



from typing import Any
from CxAdmin.api.http.httpClientModel import HTTPClientModel
from CxAdmin.objects.__cxQueue import CxQueue


class CxQueues:
    _httpClient: HTTPClientModel
    _path: str = ""

    def __init__(self, httpClient: HTTPClientModel, path: str):
        self._httpClient = httpClient
        self._path = path

    def getQueues(self) -> list[CxQueue]:
        queuesJson: list[dict[str, Any]] = self._httpClient.get(self._path).json()[
            "result"
        ]
        queues = [CxQueue.from_json(queueJson) for queueJson in queuesJson]
        return queues

    def get(self) -> list[CxQueue]:
        return self.getQueues()

    def getActiveQueues(self) -> list[CxQueue]:
        queues = self.getQueues()
        activeQueues = [queue for queue in queues if queue.active]
        return activeQueues



from CxAdmin.objects.__cxGroup import CxGroup
from CxAdmin.objects.__cxList import CxList
from CxAdmin.objects.__cxQueue import CxQueue
from CxAdmin.objects.__cxUser import CxUser
from CxAdmin.objects.__platformStatus import PlatformStatus
from CxAdmin.objects.__skill import Skill

CxGroup = CxGroup
CxList = CxList
CxQueue = CxQueue
CxUser = CxUser
PlatformStatus = PlatformStatus
Skill = Skill



from datetime import datetime
from typing import Any
from CxAdmin.api.cxItem import CxItem


class CxStatistics(CxItem):
    def getInteractions(
        self, between: tuple[datetime, datetime]
    ) -> list[dict[str, Any]]:
        params: str = f"?start={between[0]}&end={between[1]}&limit=1000"
        responsesJsons: list[dict[str, Any]] = []
        response = self._httpClient.get(f"{self._path}/interactions{params}")
        responseJson = response.json()
        responseResults = responseJson["results"]
        responsesJsons.extend(responseResults)
        while len(responsesJsons) < responseJson["total"]:
            params = f"?start={between[0]}&end={between[1]}&limit=1000&offset={len(responsesJsons)}"
            response = self._httpClient.get(f"{self._path}/interactions{params}")
            responseJson = response.json()
            responseResults = responseJson["results"]
            responsesJsons.extend(responseResults)

        return responsesJsons

    def get(self) -> Any:
        statsJson: list[dict[str, Any]] = self._httpClient.get(
            f"{self._path}/statistics"
        ).json()["result"]
        return statsJson



from CxAdmin.api.cxEnvironment import CxEnvironment
from CxAdmin.api.cxFlows import CxFlows
from CxAdmin.api.cxGroups import CxGroups
from CxAdmin.api.cxItem import CxItem
from CxAdmin.api.cxLists import CxLists
from CxAdmin.api.cxQueues import CxQueues
from CxAdmin.api.cxStatistics import CxStatistics
from CxAdmin.api.cxUsers import CxUsers

CxEnvironment = CxEnvironment
CxFlows = CxFlows
CxGroups = CxGroups
CxItem = CxItem
CxLists = CxLists
CxQueues = CxQueues
CxStatistics = CxStatistics
CxUsers = CxUsers



from typing import Any, Optional
from datetime import datetime
from CxAdmin.objects.__platformStatus import PlatformStatus
from CxAdmin.objects.__skill import Skill


class CxUser(dict[str, Any]):
    activeExtension: dict[str, str]
    additionalRoleIds: Optional[str]
    aliasPlatformUserId: Optional[str]
    clientLogLevel: Optional[str]
    created: datetime
    createdBy: str
    defaultTenant: Optional[str]
    email: str
    extensions: list[dict[str, Any]]
    externalId: Optional[str]
    firstName: str
    groups: list[dict[str, str]]
    id: str
    lastName: str
    personalTelephone: Optional[str]
    tenantStatus: PlatformStatus
    roleName: str
    skills: list[Skill]
    state: str
    platformStatus: PlatformStatus
    updated: datetime
    updatedBy: str

    def __init__(
        self,
        activeExtension: dict[str, str],
        additionalRoleIds: Optional[str],
        aliasPlatformUserId: Optional[str],
        clientLogLevel: Optional[str],
        created: datetime,
        createdBy: str,
        defaultTenant: Optional[str],
        email: str,
        extensions: list[dict[str, Any]],
        externalId: Optional[str],
        firstName: str,
        groups: list[dict[str, str]],
        id: str,
        lastName: str,
        personalTelephone: Optional[str],
        tenantStatus: PlatformStatus,
        roleName: str,
        skills: list[Skill],
        state: str,
        platformStatus: PlatformStatus,
        updated: datetime,
        updatedBy: str,
    ):
        self.activeExtension = activeExtension
        self.additionalRoleIds = additionalRoleIds
        self.aliasPlatformUserId = aliasPlatformUserId
        self.clientLogLevel = clientLogLevel
        self.created = created
        self.createdBy = createdBy
        self.defaultTenant = defaultTenant
        self.email = email
        self.extensions = extensions
        self.externalId = externalId
        self.firstName = firstName
        self.groups = groups
        self.id = id
        self.lastName = lastName
        self.personalTelephone = personalTelephone
        self.tenantStatus = tenantStatus
        self.roleName = roleName
        self.skills = skills
        self.state = state
        self.platformStatus = platformStatus
        self.updated = updated
        self.updatedBy = updatedBy

    @staticmethod
    def from_json(data: dict[str, Any]) -> "CxUser":
        return CxUser(
            activeExtension=data["activeExtension"],
            additionalRoleIds=data.get("additionalRoleIds"),
            aliasPlatformUserId=data.get("aliasPlatformUserId"),
            clientLogLevel=data["clientLogLevel"],
            created=datetime.fromisoformat(data["created"][:-1]),
            createdBy=data["createdBy"],
            defaultTenant=data.get("defaultTenant"),
            email=data["email"],
            extensions=data["extensions"],
            externalId=data["externalId"],
            firstName=data["firstName"],
            groups=data["groups"],
            id=data["id"],
            lastName=data["lastName"],
            personalTelephone=data["personalTelephone"],
            tenantStatus=PlatformStatus(data["platformStatus"]),
            roleName=data["roleName"],
            skills=[Skill.from_json(skill) for skill in data["skills"]],
            state=data["state"],
            platformStatus=PlatformStatus(data["status"]),
            updated=datetime.fromisoformat(data["updated"][:-1]),
            updatedBy=data["updatedBy"],
        )

    def __str__(self) -> str:
        return f"CxUser({self.id}, {self.email})"

    def to_json(self) -> dict[str, Any]:
        return {
            "activeExtension": self.activeExtension,
            "additionalRoleIds": self.additionalRoleIds,
            "aliasPlatformUserId": self.aliasPlatformUserId,
            "clientLogLevel": self.clientLogLevel,
            "created": self.created.isoformat(),
            "createdBy": self.createdBy,
            "defaultTenant": self.defaultTenant,
            "email": self.email,
            "extensions": self.extensions,
            "externalId": self.externalId,
            "firstName": self.firstName,
            "groups": self.groups,
            "id": self.id,
            "lastName": self.lastName,
            "personalTelephone": self.personalTelephone,
            "tenantStatus": self.tenantStatus,
            "roleName": self.roleName,
            "skills": [skill.to_json() for skill in self.skills],
            "state": self.state,
            "platformStatus": self.platformStatus,
            "updated": self.updated.isoformat(),
            "updatedBy": self.updatedBy,
        }

    def __repr__(self) -> str:
        return self.to_json().__repr__()



from typing import Any
from CxAdmin.api.cxItem import CxItem


class CxEnvironment(CxItem):
    def getTenant(self) -> Any:
        return self._httpClient.get(self._path).json()

    def getRegions(self) -> Any:
        return self._httpClient.get(f"{self._path}/regions").json()

    def get(self) -> Any:
        return self.getTenant()



from typing import Any


class CxQueue:
    """
    | Parameter | Type | Description |
    | --- | --- | --- |
    | tenantId | `UUID` | The unique identifier of the tenant with which the queue is associated. |
    | name | `string` | The name of the queue. |
    | description | `string` | The human-readable description of the queue. |
    | updated | `string` | The date and time (in UTC format) when the queue was last updated. |
    | updatedBy | `UUID` | The unique identifier of the user who last updated the queue. |
    | created | `string` | The date and time (in UTC format) when the queue was created. |
    | createdBy | `string` | The unique identifier of the user who created the queue. |
    | active | `string` | A flag indicating whether or not the queue is in use. |
    | activeVersion | `string` | The version of the queue that is in use. |
    | id | `UUID` | The ID of the queue |
    """

    def __init__(
        self,
        tenantID: str,
        name: str,
        description: str,
        updated: str,
        updatedBy: str,
        created: str,
        createdBy: str,
        active: bool,
        activeVersion: str,
        id: str,
    ):
        self.tenantID = tenantID
        self.name = name
        self.description = description
        self.updated = updated
        self.updatedBy = updatedBy
        self.created = created
        self.createdBy = createdBy
        self.active = active
        self.activeVersion = activeVersion
        self.id = id

    @staticmethod
    def from_json(data: dict[str, Any]) -> "CxQueue":
        return CxQueue(
            tenantID=data["tenantId"],
            name=data["name"],
            description=data["description"],
            updated=data["updated"],
            updatedBy=data["updatedBy"],
            created=data["created"],
            createdBy=data["createdBy"],
            active=data["active"],
            activeVersion=data["activeVersion"],
            id=data["id"],
        )

    def __str__(self) -> str:
        return f"CxQueue({self.name}, {self.id})"

    def __repr__(self) -> str:
        return self.__str__()



from typing import Any


class CxListType:
    """
    The type of list object in CxEngage.

    | Parameter | Type | Description |
    | --- | --- | --- |
    | tenantId | `UUID` | The tenant’s unique identifier. |
    | description | `string` | A description of the list type object. |
    | createdBy | `UUID` | The unique identifier of the Platform Administrator who created the list type object. |
    | updated | `string` | The date and time, in UTC format, when the list type object was last modified. |
    | name | `string` | The name of the list type. |
    | fields | `sub object` | An object containing a list of fields for the list type. For each field, you must specify the following parameters: type—The value type for the field. Value options are string, boolean, or number. The first field specified in the object must have a value of string in the type parameter. name—A unique name for the field. label—A label to be displayed in the user interface. required—A flag to indicate whether this is a required field. Valid options are true or false. The first field specified in the object must have a value of true for the required parameter. |
    | created | `string` | The date and time, in UTC format, when the list type was created. |
    | updatedBy | `UUID` | The unique identifier of the Platform Administrator who last updated the list type object. |
    | active | `string` | Whether or not the list type is enabled. Valid values are true and false. |
    | id | `UUID` | The list type object’s unique identifier. |
    """

    def __init__(
        self,
        tenantID: str,
        description: str,
        createdBy: str,
        updated: str,
        name: str,
        fields: str,
        created: str,
        updatedBy: str,
        active: str,
        id: str,
    ):
        self.tenantID = tenantID
        self.description = description
        self.createdBy = createdBy
        self.updated = updated
        self.name = name
        self.fields = fields
        self.created = created
        self.updatedBy = updatedBy
        self.active = active
        self.id = id

    @staticmethod
    def from_json(data: dict[str, Any]) -> "CxListType":
        return CxListType(
            tenantID=data["tenantId"],
            description=data["description"],
            createdBy=data["createdBy"],
            updated=data["updated"],
            name=data["name"],
            fields=data["fields"],
            created=data["created"],
            updatedBy=data["updatedBy"],
            active=data["active"],
            id=data["id"],
        )


class CxList:
    """
    A list includes the following parameters:

    | Parameter | Type | Description |
    | --- | --- | --- |
    | tenantId | `UUID` | The unique identifier of the tenant. |
    | listType | `object` |  A JSON object containing details of the list type. This parameter is only returned if you have `MANAGE_ALL_LISTS` permission. [Learn more about the content of the listType object](https://api-docs.cxengage.net/Rest/Content/List_Types/Return_list_type_object.htm#ListTypeObjectParams).
    | createdBy | `UUID` | The unique identifier of the user who created the list. |
    | listTypeId | `UUID` | The unique identifier of the list type assigned to this list. |
    | updated | `string` | The date and time, in UTC format, when the list was last modified. |
    | name | `string` | The name of the list. |
    | created | `string` | The date and time, in UTC format, when the list type was created. |
    | updatedBy | `UUID` | The unique identifier of the user who last updated the list. |
    | active | `string` | Whether or not the list is enabled. Valid values are true and false. |
    | id | `UUID` | The list's unique identifier. |
    | shared | `boolean` | Whether or not the list is shared to child tenants of this tenant. Valid values are true and false. |
    | items | `object` | The list items that make up the list, along with their unique identifiers, and (if available) a description of the list item. |
    """

    def __init__(
        self,
        tenantId: str,
        listType: CxListType,
        createdBy: str,
        listTypeID: str,
        updated: str,
        name: str,
        created: str,
        updatedBy: str,
        active: str,
        id: str,
        shared: str,
        items: list[dict[str, Any]],
    ):
        self.tenantId = tenantId
        self.listType = listType
        self.createdBy = createdBy
        self.listTypeID = listTypeID
        self.updated = updated
        self.name = name
        self.created = created
        self.updatedBy = updatedBy
        self.active = active
        self.id = id
        self.shared = shared
        self.items = items

    @staticmethod
    def from_json(data: dict[str, Any]) -> "CxList":
        return CxList(
            tenantId=data["tenantId"],
            listType=CxListType.from_json(data["listType"]),
            createdBy=data["createdBy"],
            listTypeID=data["listTypeId"],
            updated=data["updated"],
            name=data["name"],
            created=data["created"],
            updatedBy=data["updatedBy"],
            active=data["active"],
            id=data["id"],
            shared=data["shared"],
            items=data["items"],
        )

    def constructDataCSV(self, headers: bool = False) -> str:
        """
        Constructs a CSV of the data items of the CxList for reupload to the tenant.
        """
        data: list[list[str]] = []
        if headers:
            headings = list(self.items[0].keys())
            data.append(headings)

        for item in self.items:
            thisRow = list(item.values())
            data.append(thisRow)

        # convert list of lists to CSV
        csvData = ""
        for row in data:
            csvData += ",".join(row) + "\n"

        return csvData

    def __str__(self) -> str:
        return f"CxList({self.name}, {self.id})"

    def __repr__(self) -> str:
        return self.__str__()



from datetime import datetime
from typing import Any


class Skill:
    id: str
    name: str
    proficiency: int
    added: datetime

    def __init__(
        self,
        id: str,
        name: str,
        proficiency: int,
        added: datetime,
    ):
        self.id = id
        self.name = name
        self.proficiency = proficiency
        self.added = added

    @staticmethod
    def from_json(data: dict[str, Any]) -> "Skill":
        return Skill(
            id=data["id"],
            name=data["name"],
            proficiency=data["proficiency"],
            added=datetime.fromisoformat(data["added"][:-1]),
        )

    def to_json(self) -> dict[str, Any]:
        return {
            "id": self.id,
            "name": self.name,
            "proficiency": self.proficiency,
            "added": self.added.isoformat(),
        }



from enum import Enum


class PlatformStatus(Enum):
    pending = "pending"
    accepted = "accepted"
    enabled = "enabled"
    disabled = "disabled"

    def __eq__(self, __o: object) -> bool:
        if isinstance(__o, PlatformStatus):
            return self.value == __o.value
        return False



from typing import Any
from CxAdmin.api.cxItem import CxItem
from CxAdmin.objects.__cxGroup import CxGroup


class CxGroups(CxItem):
    def getGroups(self) -> list[CxGroup]:
        groupsJson: list[dict[str, Any]] = self._httpClient.get(self._path).json()[
            "result"
        ]
        groups = [CxGroup.from_json(groupJson) for groupJson in groupsJson]
        return groups

    def get(self) -> Any:
        return self.getGroups()



import abc
from typing import Any
from requests import Response


class HTTPClientModel:
    @abc.abstractmethod
    def __init__(self, basePath: str, token: str) -> None:
        ...

    @abc.abstractmethod
    def get(self, path: str) -> Response:
        ...

    @abc.abstractmethod
    def post(self, path: str, data: Any) -> Response:
        ...

    @staticmethod
    @abc.abstractmethod
    def getToken(basePath: str, apiKey: str, apiSecret: str, tenantID: str) -> str:
        ...



from CxAdmin.api.http.httpClientModel import HTTPClientModel
from abc import abstractmethod
from typing import Any


class CxItem:
    _httpClient: HTTPClientModel
    _path: str = ""

    def __init__(self, httpClient: HTTPClientModel, path: str):
        self._httpClient = httpClient
        self._path = path

    @abstractmethod
    def get(self) -> list[Any]:
        raise NotImplementedError()



from typing import Any
from CxAdmin.api.cxItem import CxItem
from CxAdmin.objects.__cxUser import CxUser


class CxUsers(CxItem):
    def getAllUsers(self) -> list[CxUser]:
        usersJson: list[dict[str, Any]] = self._httpClient.get(self._path).json()[
            "result"
        ]
        users = [CxUser.from_json(userJson) for userJson in usersJson]
        return users

    def get(self) -> Any:
        return self.getAllUsers()



from typing import Any
from CxAdmin.api.cxItem import CxItem
from CxAdmin.objects.__cxList import CxList
import json


class CxLists(CxItem):
    def get(self) -> list[CxList]:
        listsJson: list[dict[str, Any]] = self._httpClient.get(self._path).json()[
            "result"
        ]
        lists = [CxList.from_json(thisListJson) for thisListJson in listsJson]
        return lists

    def getList(self, listId: str) -> CxList:
        listJson: str = self._httpClient.get(f"{self._path}/{listId}").json()
        listItem = CxList.from_json(json.loads(listJson)["result"])
        return listItem

    def uploadListCSV(self, id: str, csv: str) -> dict[str, Any]:
        raise NotImplementedError(
            r"Not yet implemented. See https://api-docs.cxengage.net/Rest/Default.htm#Lists/bulk_Import_Lists.htm%3FTocPath%3DConfiguration%7CManaging%2520Lists%7CLists%7C_____6"
        )
        listJson: dict[str, Any] = self._httpClient.post(
            f"{self._path}/{id}/upload", csv
        )
        return listJson

    def uploadList(self, list: CxList) -> dict[str, Any]:
        return self.uploadListCSV(
            list.id,
            list.constructDataCSV(),
        )

    def getListCSV(self, listId: str) -> Any:
        return self._httpClient.get(f"{self._path}/{listId}/download/list-items.csv")



from typing import Any


class CxGroup:
    """
    A group includes the following parameters:

    | Parameter | Type | Description |
    | --- | --- | --- |
    | tenantId | `UUID` | The tenant's unique identifier. |
    | name | `string` | The name of the group. |
    | owner | `UUID` | The unique identifier of the user who owns the group. |
    | status | `boolean` | Whether or not the group is enabled. |
    | description | `string` | A description of the group. |
    | created | `string` | The date and time, in UTC format, when the group was created. |
    | createdBy | `UUID` | The unique identifier of the user who created the group. |
    | updated | `string` | The date and time, in UTC format, when the group was last modified. |
    | updatedBy | `UUID` | The unique identifier of the user who last updated the group. |
    | id | `UUID` | The group's unique identifier. |
    """

    def __init__(
        self,
        tenantId: str,
        name: str,
        owner: str,
        status: bool,
        description: str,
        created: str,
        createdBy: str,
        updated: str,
        updatedBy: str,
        id: str,
    ):
        self.tenantId = tenantId
        self.name = name
        self.owner = owner
        self.status = status
        self.description = description
        self.created = created
        self.createdBy = createdBy
        self.updated = updated
        self.updatedBy = updatedBy
        self.id = id

    @staticmethod
    def from_json(data: dict[str, Any]) -> "CxGroup":
        return CxGroup(
            tenantId=data["tenantId"],
            name=data["name"],
            owner=data["owner"],
            status=data["active"],  # Skylight's API docs are wrong, key is "active"
            description=data["description"],
            created=data["created"],
            createdBy=data["createdBy"],
            updated=data["updated"],
            updatedBy=data["updatedBy"],
            id=data["id"],
        )

    def __str__(self) -> str:
        return f"CxGroup({self.name}, {self.id})"

    def __repr__(self) -> str:
        return self.__str__()



from CxAdmin.api.http.httpClientModel import HTTPClientModel
from typing import Any, Optional
from requests import get, post, Response


class HTTPClient(HTTPClientModel):
    __basePath: str
    __token: str
    __headers: dict[str, Any]

    def __init__(self, basePath: str, token: str):
        self.__basePath = basePath
        self.__token = token
        self.__headers = {
            "Authorization": f"Token {self.__token}",
            "Content-Type": "application/json",
        }

    def get(self, path: str) -> Response:
        response = HTTPClient._get(
            path=self.__basePath + path,
            headers=self.__headers,
        )
        return response

    def post(self, path: str, data: Any) -> Response:
        response = HTTPClient._post(
            path=self.__basePath + path,
            headers=self.__headers,
            body=data,
        )
        return response

    @staticmethod
    def _get(path: str, headers: dict[str, Any]) -> Response:
        response = get(path, headers=headers)
        return response

    @staticmethod
    def _post(
        path: str,
        body: Optional[dict[str, Any]],
        headers: Optional[dict[str, Any]] = None,
    ) -> Response:
        response = post(url=path, json=body, headers=headers)
        return response

    @staticmethod
    def getToken(basePath: str, apiKey: str, apiSecret: str, tenantID: str) -> str:
        body = {
            "username": apiKey,
            "password": apiSecret,
            "tenantId": tenantID,
        }

        response = HTTPClient._post(
            path=f"{basePath}/v1/tokens",
            body=body,
        )

        token: str = response.json()["token"]
        return token
